import processing.core.PApplet;
import toxi.geom.*;
import toxi.math.MathUtils;
import toxi.processing.ToxiclibsSupport;

public class Segment extends BoilerPlate {
	PApplet parent;

	public int seed = 0;
	public float seedNormal = 0.0f;

	public Ray3D currentRotator;
	public Ray3D currentArbitrary;
	public Ray3D currentTangent;

	public Vec3D currentDirection;
	public Ray3D current;

	public Vec3D previousDirection;
	public Ray3D previous;

	public Segment parentSegment;
	public float segmentScalar = 10.0f;

	public int frameCount;
	public float frameCountScalar;
	public float rotationScalar;
	public float seedNormalScalar;
	public float layerRadiusOffset;
	public float layerRadiusScalar;
	public float frameCountSpeedOffset;
	public float frameCountSpeedNormal;
    
	Segment(PApplet p) {
		parent = p;
		setScope("Segment");
		currentDirection = new Vec3D();
		current = new Ray3D();
		previousDirection = new Vec3D();
		previous = new Ray3D();

		currentArbitrary = new Ray3D(new Vec3D(),Vec3D.Y_AXIS);
		currentTangent = new Ray3D();
		currentRotator = new Ray3D();
		
	}

	public void randomizeCurrentControls() {
		float r;
		r = 1.0f;
		currentDirection = new Vec3D();
		r = 100.0f;
		current = new Ray3D();
	}

	public void setPreviousControls() {
		previousDirection = parentSegment.currentDirection;
		previous = parentSegment.current;
	}


	public void parse(ToxiclibsSupport gfx) {
		previousDirection.set(parentSegment.currentDirection);
		previous.set(parentSegment.current);

		current.set(current.sub(previous));
		current.normalizeTo(segmentScalar);
		current.addSelf(previous);
		current.setDirection(previous.sub(current));

		currentArbitrary.set(current);
		currentTangent.set(current);
		currentRotator.set(current);
		
	}

	public void draw(ToxiclibsSupport gfx) {
		// tracer("draw");
		parent.noFill();


		float colorAlpha = (1-seedNormal) * 255.0f + 0.0f;

		setLineStyle(1,0xFFFF6600,(1-seedNormal));
		gfx.ray(current,segmentScalar);

		int wings = 2;
		int layers = 20;

		Vec3D currentVec = new Vec3D(current);
        Vec3D dirVec = new Vec3D(current.getDirection());
        Vec3D crossVec = currentArbitrary.cross(dirVec.getNormalized()).normalize();

        for (int i = 0; i < wings; i++) {
        	// convert positions
	        float wingPercentage = (float)i/(float)wings;
	        float angleInRads = MathUtils.TWO_PI*wingPercentage;
	        angleInRads += seedNormal*seedNormalScalar;	// extra offset of rotation
	        angleInRads += frameCount*frameCountScalar;	// speed of rotation

	        float frameCountSpeed = frameCountSpeedNormal + seedNormal*frameCountSpeedOffset*.1f;
	        float frameCountAngle = 90.0f*(float)(i*2-1);
	        float rangeOffsetInRads = MathUtils.sin(frameCount*frameCountSpeed)*frameCountAngle/360.0f*MathUtils.PI;
	        
	        angleInRads += rangeOffsetInRads;
	        
	        // create local rotation vector
	        Vec3D rotVec = crossVec.getRotatedAroundAxis(dirVec, angleInRads);
	        
	        // currentRotator.setDirection(rotVec);
	        // setLineStyle(1,0xFFFF00FF,(1-seedNormal));
			// gfx.ray(currentRotator,rotationScalar);
	        
	        for (int j = 0; j < layers; j++) {
		        float layerPercentage = (float)j/(float)layers;
		        float layerRadius = layerRadiusScalar*layerPercentage;	// extra offset of rotation
		        layerRadius += layerRadiusOffset;
				Vec3D point = new Vec3D(rotVec);
				point.scaleSelf(layerRadius);
				point.addSelf(currentVec);
		        setLineStyle(1,0xFFFFFF00,(1-seedNormal)*(1-layerPercentage));
				gfx.point(point);
	        }

        }

	}

	public void setLineStyle(int weightParam,int colorParam,float alphaParam){
		parent.noFill();
		parent.strokeWeight(weightParam);
		parent.stroke(colorParam, 255.0f*alphaParam);

	}
}
